#include <QQuickWidget>
#include <QQuickView>
#include <QQmlError>
#include <QApplication>
#include <QMainWindow>
#include <QLabel>
#include <QGridLayout>
#include <QDebug>
#include <QMenu>
#include <QMenuBar>
#include <QToolBar>
#include <QAction>
#include <QTimer>
#ifdef QT_DIAG_LIB
# include <qwindowdump.h>
#endif

bool optUseWindowContainer = true;
bool optUseQuickWidget = true;

class MainWindow : public QMainWindow {
    Q_OBJECT
public:
    MainWindow();

public slots:
    void dumpWindows();
};

void MainWindow::dumpWindows()
{
#ifdef QT_DIAG_LIB
    static  int n =0;
    qDebug() << "\n#" << ++n;
    QtDiag::dumpAllWindows();
#endif
}

MainWindow::MainWindow()
{
    setObjectName("Main");

    QMenu *mn = menuBar()->addMenu("File");
    QToolBar *tb = new QToolBar("File");

    mn->setObjectName("MainMenu");
    QAction *a = mn->addAction("Dump");
    tb->addAction(a);
    a->setShortcut(Qt::CTRL + Qt::Key_D);    
    connect(a, &QAction::triggered,
            this, &MainWindow::dumpWindows);
    a = mn->addAction("Quit");
    tb->addAction(a);
    a->setShortcut(Qt::CTRL + Qt::Key_Q);
    connect(a, &QAction::triggered, this, &QWidget::close);
    addToolBar(Qt::TopToolBarArea, tb);

    QUrl source("qrc:qtbug38116/main.qml");

    int status = QQuickWidget::Error;

    setWindowTitle(QT_VERSION_STR);

    QWidget *cw = new QWidget;
    setCentralWidget(cw);
    QGridLayout *grid = new QGridLayout(cw);
    int column = 0;
    if (optUseQuickWidget) {
        grid->addWidget(new QLabel("QuickWidget"), 0, column);
        QQuickWidget *quickWidget = new QQuickWidget;
        quickWidget->setMinimumSize(200, 200);
        quickWidget->setResizeMode(QQuickWidget::SizeRootObjectToView );
        quickWidget->setSource(source);
        quickWidget->setObjectName("QuickWidget");
        status = quickWidget->status();
        grid->addWidget(quickWidget, 1, column);
        column++;
    }
    if (optUseWindowContainer) {
        grid->addWidget(new QLabel("createWindowContainer"), 0, column);
        QQuickView *quickView = new QQuickView(source);
        quickView->resize(200,200);
        quickView->setResizeMode(QQuickView::SizeRootObjectToView );
        quickView->setObjectName("QQuickView");
        QWidget *container = QWidget::createWindowContainer(quickView, Q_NULLPTR, Qt::Widget);
        container->setObjectName("container");
        container->setMinimumSize(200, 200);
        grid->addWidget(container, 1, column);
        status =  quickView->status();
    }
    qDebug() << source << static_cast<QQuickView::Status>(status);
}

int main(int argc, char **argv)
{
    QApplication app(argc, argv);
    QStringList args = app.arguments();
    args.pop_front();

    qDebug() << QT_VERSION_STR << args;

    optUseWindowContainer = !args.contains("-c");
    optUseQuickWidget = !args.contains("-q");

    MainWindow mainWindow;
    mainWindow.move(400, 400);
    mainWindow.show();
#ifdef QT_DIAG_LIB
    if (args.contains("-t")) {
        QTimer *t = new QTimer(&mainWindow);
        QObject::connect(t, &QTimer::timeout, &mainWindow, &MainWindow::dumpWindows);

        t->start(2000);
    }
#endif
    return app.exec();
}

#include "main.moc"
